#!/usr/bin/env python
# Copyright (c) 2000-2006 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

"""Komodo Logging system service"""

import os
import types
import logging

from xpcom import components, nsError, ServerException, COMException, server

import timeline
timeline.enter('importing koLoggingService')


#---- globals

_loggerNames = [''] #XXX is root logger getting added again later?



#---- mods to the Python logging system

class LoggerWithNotify(logging.Logger):
    def __init__(self, *args, **kwargs):
        logging.Logger.__init__(self, *args, **kwargs)
        try:
            self._observerSvc = components.classes["@mozilla.org/observer-service;1"]\
                                .getService(components.interfaces.nsIObserverService)
        except:
            print "WARN: failed to get Observer Service with creating "\
                  "LoggerWithNotify"
        else:
            try:
                self._observerSvc.notifyObservers(None, 'add_logger',
                                                  self.name)
            except COMException:
                pass
        _loggerNames.append(self.name)

    def setLevel(self, level):
        try:
            logging.Logger.setLevel(self, level)
            if hasattr(self, '_observerSvc'):
                try:
                    self._observerSvc.notifyObservers(None,
                            "set_logger_level",
                            "%s:%d" % (self.name, level))
                except COMException:
                    pass
        except:
            import traceback
            traceback.print_last()
            raise
logging.setLoggerClass(LoggerWithNotify)


#---- XPCOM logging service

class KoLoggingService:
    """A service from which JavaScript code can use the Python logging
    infrastructure.
    """
    _com_interfaces_ = [components.interfaces.koILoggingService]
    _reg_clsid_ = "{5DBBF27D-F084-432B-9422-9C9B4707452D}"
    _reg_contractid_ = "@activestate.com/koLoggingService;1"
    _reg_desc_ = "Komodo Logging system service"

    def __init__(self):
        timeline.enter('koLoggingService.__init__')
        hdlr = logging.StreamHandler()
        fmt = logging.Formatter("%(levelname)s: %(name)s: %(message)s")
        hdlr.setFormatter(fmt)
        logging.root.addHandler(hdlr)
        kohdlr = NotifyingLogHandler()
        kohdlr.setFormatter(logging.Formatter("%(name)s"))
        logging.root.addHandler(kohdlr)
        timeline.leave('koLoggingService.__init__')
        
    def getLogger(self, logger_name):
        timeline.startTimer('koLoggingService.getLogger')
        logger = KoLogger(logger_name)
        logger = server.WrapObject(logger, components.interfaces.koILogger)
        timeline.stopTimer('koLoggingService.getLogger')
        timeline.markTimer('koLoggingService.getLogger')
        return logger
     
    def getLoggerNames(self):
        _loggerNames.sort()
        return _loggerNames


class KoLogRecord:
    _com_interfaces_ = [components.interfaces.koILogRecord]
    _reg_clsid_ = "{46AF3809-D4A4-4649-84F6-2EB9D2CFB842}"
    _reg_contractid_ = "@activestate.com/koLogRecord;1"
    _reg_desc_ = "Komodo Log Record"


class NotifyingLogHandler(logging.Handler):
    """A handler that passes each log message to the Mozilla notification
    system: 'emit_log'.
    
    This is used by the GUI logging control window.
    """
    def __init__(self, *args, **kwargs):
        logging.Handler.__init__(self, *args, **kwargs)
        self._observerSvc = components.classes["@mozilla.org/observer-service;1"]\
                    .getService(components.interfaces.nsIObserverService)
        
    def emit(self, record):
        korecord = KoLogRecord()
        # XXX for some reason, format doesn't give us the message
        # korecord.message = self.format(record)
        korecord.message = record.getMessage()
        korecord.levelname = record.levelname
        korecord.logger = record.name
        self._observerSvc.notifyObservers(korecord, 'emit_log', '')
        

class KoLogger:
    """This is a light XPCOM proxy for a real Logger class in the Python
    logging system.
    """
    _com_interfaces_ = [components.interfaces.koILogger]
    _reg_clsid_ = "{EE4DC45E-1BE7-4C3C-B521-1434FA1CF054}"
    _reg_contractid_ = "@activestate.com/koLogger;1"
    _reg_desc_ = "Komodo Logger"

    def __init__(self, name):
        timeline.startTimer('KoLogger.__init__')
        self._logger = logging.getLogger(name)
        timeline.stopTimer('KoLogger.__init__')
        timeline.markTimer('KoLogger.__init__')

    def setLevel(self, level):
        #XXX name=='root' seems wrong. The root logger's name is the empty
        #    string. David? --TM
        if level == 0 and self._logger.name == 'root': # root logger shouldn't be unset ever
            self._logger.setLevel(logging.WARN)
        else:
            self._logger.setLevel(level)
        
    def getEffectiveLevel(self):
        return self._logger.getEffectiveLevel()

    def get_level(self):
        return self._logger.level

    def isEnabledFor(self, level):
        return self._logger.isEnabledFor(level)
    
    def debug(self, msg):
        return self._logger.debug(msg)

    def info(self, msg):
        return self._logger.info(msg)

    def warn(self, msg):
        return self._logger.warn(msg)

    def error(self, msg):
        return self._logger.error(msg)

    def critical(self, msg):
        return self._logger.critical(msg)


timeline.leave('importing koLoggingService')
